"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metric import Metric, MetricTypedDict
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MetricsTypedDict(TypedDict):
    orders: MetricTypedDict
    r"""Information about a metric."""
    revenue: MetricTypedDict
    r"""Information about a metric."""
    net_revenue: MetricTypedDict
    r"""Information about a metric."""
    cumulative_revenue: MetricTypedDict
    r"""Information about a metric."""
    net_cumulative_revenue: MetricTypedDict
    r"""Information about a metric."""
    average_order_value: MetricTypedDict
    r"""Information about a metric."""
    net_average_order_value: MetricTypedDict
    r"""Information about a metric."""
    one_time_products: MetricTypedDict
    r"""Information about a metric."""
    one_time_products_revenue: MetricTypedDict
    r"""Information about a metric."""
    one_time_products_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    new_subscriptions_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions_revenue: MetricTypedDict
    r"""Information about a metric."""
    renewed_subscriptions_net_revenue: MetricTypedDict
    r"""Information about a metric."""
    active_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    monthly_recurring_revenue: MetricTypedDict
    r"""Information about a metric."""
    committed_monthly_recurring_revenue: MetricTypedDict
    r"""Information about a metric."""
    checkouts: MetricTypedDict
    r"""Information about a metric."""
    succeeded_checkouts: MetricTypedDict
    r"""Information about a metric."""
    checkouts_conversion: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_customer_service: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_low_quality: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_missing_features: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_switched_service: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_too_complex: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_too_expensive: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_unused: MetricTypedDict
    r"""Information about a metric."""
    canceled_subscriptions_other: MetricTypedDict
    r"""Information about a metric."""


class Metrics(BaseModel):
    orders: Metric
    r"""Information about a metric."""

    revenue: Metric
    r"""Information about a metric."""

    net_revenue: Metric
    r"""Information about a metric."""

    cumulative_revenue: Metric
    r"""Information about a metric."""

    net_cumulative_revenue: Metric
    r"""Information about a metric."""

    average_order_value: Metric
    r"""Information about a metric."""

    net_average_order_value: Metric
    r"""Information about a metric."""

    one_time_products: Metric
    r"""Information about a metric."""

    one_time_products_revenue: Metric
    r"""Information about a metric."""

    one_time_products_net_revenue: Metric
    r"""Information about a metric."""

    new_subscriptions: Metric
    r"""Information about a metric."""

    new_subscriptions_revenue: Metric
    r"""Information about a metric."""

    new_subscriptions_net_revenue: Metric
    r"""Information about a metric."""

    renewed_subscriptions: Metric
    r"""Information about a metric."""

    renewed_subscriptions_revenue: Metric
    r"""Information about a metric."""

    renewed_subscriptions_net_revenue: Metric
    r"""Information about a metric."""

    active_subscriptions: Metric
    r"""Information about a metric."""

    monthly_recurring_revenue: Metric
    r"""Information about a metric."""

    committed_monthly_recurring_revenue: Metric
    r"""Information about a metric."""

    checkouts: Metric
    r"""Information about a metric."""

    succeeded_checkouts: Metric
    r"""Information about a metric."""

    checkouts_conversion: Metric
    r"""Information about a metric."""

    canceled_subscriptions: Metric
    r"""Information about a metric."""

    canceled_subscriptions_customer_service: Metric
    r"""Information about a metric."""

    canceled_subscriptions_low_quality: Metric
    r"""Information about a metric."""

    canceled_subscriptions_missing_features: Metric
    r"""Information about a metric."""

    canceled_subscriptions_switched_service: Metric
    r"""Information about a metric."""

    canceled_subscriptions_too_complex: Metric
    r"""Information about a metric."""

    canceled_subscriptions_too_expensive: Metric
    r"""Information about a metric."""

    canceled_subscriptions_unused: Metric
    r"""Information about a metric."""

    canceled_subscriptions_other: Metric
    r"""Information about a metric."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookeventtype import WebhookEventType
from .webhookformat import WebhookFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class WebhookEndpointTypedDict(TypedDict):
    r"""A webhook endpoint."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    url: str
    r"""The URL where the webhook events will be sent."""
    format_: WebhookFormat
    secret: str
    r"""The secret used to sign the webhook events."""
    organization_id: str
    r"""The organization ID associated with the webhook endpoint."""
    events: List[WebhookEventType]
    r"""The events that will trigger the webhook."""


class WebhookEndpoint(BaseModel):
    r"""A webhook endpoint."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    url: str
    r"""The URL where the webhook events will be sent."""

    format_: Annotated[WebhookFormat, pydantic.Field(alias="format")]

    secret: str
    r"""The secret used to sign the webhook events."""

    organization_id: str
    r"""The organization ID associated with the webhook endpoint."""

    events: List[WebhookEventType]
    r"""The events that will trigger the webhook."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customercustomermeter import CustomerCustomerMeter, CustomerCustomerMeterTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceCustomerCustomerMeterTypedDict(TypedDict):
    items: List[CustomerCustomerMeterTypedDict]
    pagination: PaginationTypedDict


class ListResourceCustomerCustomerMeter(BaseModel):
    items: List[CustomerCustomerMeter]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customer import Customer, CustomerTypedDict
from .meter import Meter, MeterTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class CustomerMeterTypedDict(TypedDict):
    r"""An active customer meter, with current consumed and credited units."""

    id: str
    r"""The ID of the object."""
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    customer_id: str
    r"""The ID of the customer."""
    meter_id: str
    r"""The ID of the meter."""
    consumed_units: float
    r"""The number of consumed units."""
    credited_units: int
    r"""The number of credited units."""
    balance: float
    r"""The balance of the meter, i.e. the difference between credited and consumed units."""
    customer: CustomerTypedDict
    r"""A customer in an organization."""
    meter: MeterTypedDict


class CustomerMeter(BaseModel):
    r"""An active customer meter, with current consumed and credited units."""

    id: str
    r"""The ID of the object."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    customer_id: str
    r"""The ID of the customer."""

    meter_id: str
    r"""The ID of the meter."""

    consumed_units: float
    r"""The number of consumed units."""

    credited_units: int
    r"""The number of credited units."""

    balance: float
    r"""The balance of the meter, i.e. the difference between credited and consumed units."""

    customer: Customer
    r"""A customer in an organization."""

    meter: Meter

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from polar_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union, cast


class PolarCustomers(BaseSDK):
    def get(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersGetSecurity,
            models.CustomerPortalCustomersGetSecurityTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPortalCustomer:
        r"""Get Customer

        Get authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/customers/me",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerPortalCustomer, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersGetSecurity,
            models.CustomerPortalCustomersGetSecurityTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPortalCustomer:
        r"""Get Customer

        Get authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/customers/me",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersGetSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:get",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerPortalCustomer, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def update(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersUpdateSecurity,
            models.CustomerPortalCustomersUpdateSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPortalCustomerUpdate,
            models.CustomerPortalCustomerUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPortalCustomer:
        r"""Update Customer

        Update authenticated customer.

        **Scopes**: `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPortalCustomerUpdate)
        request = cast(models.CustomerPortalCustomerUpdate, request)

        req = self._build_request(
            method="PATCH",
            path="/v1/customer-portal/customers/me",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersUpdateSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPortalCustomerUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:update",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerPortalCustomer, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersUpdateSecurity,
            models.CustomerPortalCustomersUpdateSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPortalCustomerUpdate,
            models.CustomerPortalCustomerUpdateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPortalCustomer:
        r"""Update Customer

        Update authenticated customer.

        **Scopes**: `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPortalCustomerUpdate)
        request = cast(models.CustomerPortalCustomerUpdate, request)

        req = self._build_request_async(
            method="PATCH",
            path="/v1/customer-portal/customers/me",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersUpdateSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPortalCustomerUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:update",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.CustomerPortalCustomer, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def list_payment_methods(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersListPaymentMethodsSecurity,
            models.CustomerPortalCustomersListPaymentMethodsSecurityTypedDict,
        ],
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CustomerPortalCustomersListPaymentMethodsResponse]:
        r"""List Customer Payment Methods

        Get saved payment methods of the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalCustomersListPaymentMethodsRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/v1/customer-portal/customers/me/payment-methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersListPaymentMethodsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:list_payment_methods",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> (
            Optional[models.CustomerPortalCustomersListPaymentMethodsResponse]
        ):
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list_payment_methods(
                security=security,
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CustomerPortalCustomersListPaymentMethodsResponse(
                result=unmarshal_json_response(
                    models.ListResourceCustomerPaymentMethod, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_payment_methods_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersListPaymentMethodsSecurity,
            models.CustomerPortalCustomersListPaymentMethodsSecurityTypedDict,
        ],
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CustomerPortalCustomersListPaymentMethodsResponse]:
        r"""List Customer Payment Methods

        Get saved payment methods of the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalCustomersListPaymentMethodsRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/customer-portal/customers/me/payment-methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersListPaymentMethodsSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:list_payment_methods",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> (
            Optional[models.CustomerPortalCustomersListPaymentMethodsResponse]
        ):
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list_payment_methods(
                security=security,
                page=next_page,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CustomerPortalCustomersListPaymentMethodsResponse(
                result=unmarshal_json_response(
                    models.ListResourceCustomerPaymentMethod, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def add_payment_method(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersAddPaymentMethodSecurity,
            models.CustomerPortalCustomersAddPaymentMethodSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPaymentMethodCreate,
            models.CustomerPaymentMethodCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPaymentMethodCreateResponse:
        r"""Add Customer Payment Method

        Add a payment method to the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPaymentMethodCreate)
        request = cast(models.CustomerPaymentMethodCreate, request)

        req = self._build_request(
            method="POST",
            path="/v1/customer-portal/customers/me/payment-methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersAddPaymentMethodSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPaymentMethodCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:add_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.CustomerPaymentMethodCreateResponse, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def add_payment_method_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersAddPaymentMethodSecurity,
            models.CustomerPortalCustomersAddPaymentMethodSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPaymentMethodCreate,
            models.CustomerPaymentMethodCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPaymentMethodCreateResponse:
        r"""Add Customer Payment Method

        Add a payment method to the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPaymentMethodCreate)
        request = cast(models.CustomerPaymentMethodCreate, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/customer-portal/customers/me/payment-methods",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersAddPaymentMethodSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPaymentMethodCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:add_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.CustomerPaymentMethodCreateResponse, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def confirm_payment_method(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersConfirmPaymentMethodSecurity,
            models.CustomerPortalCustomersConfirmPaymentMethodSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPaymentMethodConfirm,
            models.CustomerPaymentMethodConfirmTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPaymentMethodCreateResponse:
        r"""Confirm Customer Payment Method

        Confirm a payment method for the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPaymentMethodConfirm)
        request = cast(models.CustomerPaymentMethodConfirm, request)

        req = self._build_request(
            method="POST",
            path="/v1/customer-portal/customers/me/payment-methods/confirm",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersConfirmPaymentMethodSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPaymentMethodConfirm
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:confirm_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.CustomerPaymentMethodCreateResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.CustomerNotReadyData, http_res
            )
            raise models.CustomerNotReady(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def confirm_payment_method_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersConfirmPaymentMethodSecurity,
            models.CustomerPortalCustomersConfirmPaymentMethodSecurityTypedDict,
        ],
        request: Union[
            models.CustomerPaymentMethodConfirm,
            models.CustomerPaymentMethodConfirmTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomerPaymentMethodCreateResponse:
        r"""Confirm Customer Payment Method

        Confirm a payment method for the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.CustomerPaymentMethodConfirm)
        request = cast(models.CustomerPaymentMethodConfirm, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/customer-portal/customers/me/payment-methods/confirm",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersConfirmPaymentMethodSecurity
            ),
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomerPaymentMethodConfirm
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:confirm_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.CustomerPaymentMethodCreateResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.CustomerNotReadyData, http_res
            )
            raise models.CustomerNotReady(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def delete_payment_method(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersDeletePaymentMethodSecurity,
            models.CustomerPortalCustomersDeletePaymentMethodSecurityTypedDict,
        ],
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Customer Payment Method

        Delete a payment method from the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalCustomersDeletePaymentMethodRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/v1/customer-portal/customers/me/payment-methods/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersDeletePaymentMethodSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:delete_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentMethodInUseByActiveSubscriptionData, http_res
            )
            raise models.PaymentMethodInUseByActiveSubscription(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def delete_payment_method_async(
        self,
        *,
        security: Union[
            models.CustomerPortalCustomersDeletePaymentMethodSecurity,
            models.CustomerPortalCustomersDeletePaymentMethodSecurityTypedDict,
        ],
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete Customer Payment Method

        Delete a payment method from the authenticated customer.

        **Scopes**: `customer_portal:read` `customer_portal:write`

        :param security:
        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomerPortalCustomersDeletePaymentMethodRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v1/customer-portal/customers/me/payment-methods/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, models.CustomerPortalCustomersDeletePaymentMethodSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="customer_portal:customers:delete_payment_method",
                oauth2_scopes=None,
                security_source=security,
            ),
            request=req,
            error_status_codes=["400", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentMethodInUseByActiveSubscriptionData, http_res
            )
            raise models.PaymentMethodInUseByActiveSubscription(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpricemeter import ProductPriceMeter, ProductPriceMeterTypedDict
from .productpricetype import ProductPriceType
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ProductPriceMeteredUnitTypedDict(TypedDict):
    r"""A metered, usage-based, price for a product, with a fixed unit price."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the price."""
    is_archived: bool
    r"""Whether the price is archived and no longer available."""
    product_id: str
    r"""The ID of the product owning the price."""
    type: ProductPriceType
    recurring_interval: Nullable[SubscriptionRecurringInterval]
    price_currency: str
    r"""The currency."""
    unit_amount: str
    r"""The price per unit in cents."""
    cap_amount: Nullable[int]
    r"""The maximum amount in cents that can be charged, regardless of the number of units consumed."""
    meter_id: str
    r"""The ID of the meter associated to the price."""
    meter: ProductPriceMeterTypedDict
    r"""A meter associated to a metered price."""
    amount_type: Literal["metered_unit"]


class ProductPriceMeteredUnit(BaseModel):
    r"""A metered, usage-based, price for a product, with a fixed unit price."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the price."""

    is_archived: bool
    r"""Whether the price is archived and no longer available."""

    product_id: str
    r"""The ID of the product owning the price."""

    type: ProductPriceType

    recurring_interval: Annotated[
        Nullable[SubscriptionRecurringInterval],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]

    price_currency: str
    r"""The currency."""

    unit_amount: str
    r"""The price per unit in cents."""

    cap_amount: Nullable[int]
    r"""The maximum amount in cents that can be charged, regardless of the number of units consumed."""

    meter_id: str
    r"""The ID of the meter associated to the price."""

    meter: ProductPriceMeter
    r"""A meter associated to a metered price."""

    AMOUNT_TYPE: Annotated[
        Annotated[
            Literal["metered_unit"], AfterValidator(validate_const("metered_unit"))
        ],
        pydantic.Field(alias="amount_type"),
    ] = "metered_unit"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "recurring_interval", "cap_amount"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from polar_sdk.customer_session_sdk import CustomerSessionSDK
from polar_sdk.downloadables import Downloadables
from polar_sdk.polar_benefit_grants import PolarBenefitGrants
from polar_sdk.polar_customer_meters import PolarCustomerMeters
from polar_sdk.polar_customers import PolarCustomers
from polar_sdk.polar_license_keys import PolarLicenseKeys
from polar_sdk.polar_orders import PolarOrders
from polar_sdk.polar_organizations import PolarOrganizations
from polar_sdk.polar_subscriptions import PolarSubscriptions
from polar_sdk.seats import Seats
from typing import Optional


class CustomerPortal(BaseSDK):
    benefit_grants: PolarBenefitGrants
    customers: PolarCustomers
    customer_meters: PolarCustomerMeters
    seats: Seats
    customer_session: CustomerSessionSDK
    downloadables: Downloadables
    license_keys: PolarLicenseKeys
    orders: PolarOrders
    organizations: PolarOrganizations
    subscriptions: PolarSubscriptions

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.benefit_grants = PolarBenefitGrants(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.customers = PolarCustomers(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.customer_meters = PolarCustomerMeters(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.seats = Seats(self.sdk_configuration, parent_ref=self.parent_ref)
        self.customer_session = CustomerSessionSDK(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.downloadables = Downloadables(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.license_keys = PolarLicenseKeys(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.orders = PolarOrders(self.sdk_configuration, parent_ref=self.parent_ref)
        self.organizations = PolarOrganizations(
            self.sdk_configuration, parent_ref=self.parent_ref
        )
        self.subscriptions = PolarSubscriptions(
            self.sdk_configuration, parent_ref=self.parent_ref
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .customerportaloauthaccount import (
    CustomerPortalOAuthAccount,
    CustomerPortalOAuthAccountTypedDict,
)
from .taxidformat import TaxIDFormat
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CustomerPortalCustomerTaxIDTypedDict = TypeAliasType(
    "CustomerPortalCustomerTaxIDTypedDict", Union[str, TaxIDFormat]
)


CustomerPortalCustomerTaxID = TypeAliasType(
    "CustomerPortalCustomerTaxID", Union[str, TaxIDFormat]
)


class CustomerPortalCustomerTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    email: str
    email_verified: bool
    name: Nullable[str]
    billing_name: Nullable[str]
    billing_address: Nullable[AddressTypedDict]
    tax_id: Nullable[List[Nullable[CustomerPortalCustomerTaxIDTypedDict]]]
    oauth_accounts: Dict[str, CustomerPortalOAuthAccountTypedDict]
    default_payment_method_id: NotRequired[Nullable[str]]


class CustomerPortalCustomer(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    email: str

    email_verified: bool

    name: Nullable[str]

    billing_name: Nullable[str]

    billing_address: Nullable[Address]

    tax_id: Nullable[List[Nullable[CustomerPortalCustomerTaxID]]]

    oauth_accounts: Dict[str, CustomerPortalOAuthAccount]

    default_payment_method_id: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["default_payment_method_id"]
        nullable_fields = [
            "modified_at",
            "name",
            "billing_name",
            "billing_address",
            "tax_id",
            "default_payment_method_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
